/*
    Copyright (c) 2015 digitalSTROM AG, Zurich, Switzerland

    Author: Sergey 'Jin' Bostandzhyan <jin@dev.digitalstrom.org>

    This file is part of the libdSvDC UI add-on.

    libdsvdc add-on is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    libdsvdc add-on is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/

var g_vdcs = {};
var g_timeout_id;

var vdc_config =
{
    model_uid:              "libdsvdc:reference", // identifier of our vDCs
    search_intervall_ms:    20000,
    tip_1x:                 "0u",                 // ButtonInputState
    tip_2x:                 "1u",
    tip_3x:                 "2u",
    tip_4x:                 "3u"
};

var vdc_render_no_devices = function()
{
    $("#device_list").hide();
    $("#empty").text("No libdSvDC reference application devices found, check " +
                     "if your vDC is running.");
};

var vdc_clear_search_timeout = function()
{
    if (g_timeout_id !== undefined)
    {
        window.clearTimeout(g_timeout_id);
    }
};

var vdc_start_search_timeout = function()
{
    g_timeout_id = window.setTimeout(vdc_search,
                                     vdc_config.search_intervall_ms);
};


// show dialog box with optional "reload" button
var vdc_show_message = function(title, message, reload)
{
    $("#modal_dialog_title").html(title);

    if (reload === true)
    {
        message = message +
            "<div class=\"right\"><p><button type=\"button\" id=\"reload\">Reload</button></p></div>";
    }
    $("#modal_dialog_message").html(message);
  
    if (reload === true)
    {
        $("#reload").click(
            function(event)
            {
                event.preventDefault();
                window.location.hash = "";
                window.location.reload();
                return;
            });
    }

    window.location.hash = "#open_modal_dialog";
};

// expects string of type something_value_something and retrieve "value"
var vdc_button_value_from_id = function(id)
{
    var extract = String(id).match(/_(.*)_/);
    if (extract)
    {
        return extract.pop();
    }

    return null;
};

// generate button id's of type btn_somevalue_dsuid, optionally prepend with "#"
var vdc_button_id = function(dsuid, id, hash)
{
    var h = "";
    if (hash)
    {
        h = "#";
    }
    return h + "btn_" + String(id) + "_" + String(dsuid);
};

// generate td containing a button with a given text and value
var vdc_button = function(dsuid, value, text)
{
    return "<td><button id=\"" + vdc_button_id(dsuid, value) +
               "\" type=\"button\">" + text + "</button></td>";
};

// generic function that will parse incoming data and show an error box
var vdc_request_failed = function(data)
{
    var err = data.statusText;
    if (data.responseJSON && (data.responseJSON.ok === false))
    {
        err = data.responseJSON.message;
    }
    vdc_show_message("Error", "Request failed: " + err, true);
};

var vdc_render_remove_device = function(dsuid)
{
       $("#" + dsuid).remove();
};

// determine what button was clicked and send a setProperty request to the vDC
var vdc_button_clicked = function(dsuid, evt)
{
    var device = g_vdcs[dsuid];
    if (device === undefined)
    {
        vdc_show_message("Error", "Could not find device " +
                         device.DisplayID + "!<br/>" +
                         "Something went wrong, please reload the page", true);
    }

    // parse out click type from the button id
    var clickvalue = vdc_button_value_from_id(evt.target.id);
    if (clickvalue === null)
    {
        return;
    }

    var callback = function(data)
    {
        var i;
        if (data.result && data.result.GenericResponse)
        {
            var message = "";
            var code = 0;

            var res = data.result.GenericResponse;
            for (i = 0; i < res.length; i++)
            {
                var r = res[i];
                if (r.result)
                {
                    message = r.result;
                }
                else if (r.response)
                {
                    code = r.response;
                }
            }

            if (code !== 0)
            {
                vdc_show_message("Error", "The vDC was not happy with our " +
                                 "request and said: <p>" + message + "</p>");
            }
        }
    };

    $.getJSON("/json/property/vdcquery?query=setProperty(deviceId=" +
              dsuid + ")/(x-libdsvdc-actionclick=" + clickvalue + ")",
              callback).fail(vdc_request_failed);
};

// render table row containing some device information and action buttons
var vdc_render_add_device = function(dsuid)
{
   var device = g_vdcs[dsuid];
   if (device === undefined)
   {
       return;
   }

   vdc_render_remove_device(dsuid);

   var row = "<tr id=\"" + dsuid + "\"><td>" +
             "<img src=\"/icons/getDeviceIcon?dsuid=" +
             dsuid + "\"/></td><td>" +
             device.name + "</td><td>" +
             device.DisplayID + "</td>" +
             vdc_button(dsuid, vdc_config.tip_1x, "1x click") +
             vdc_button(dsuid, vdc_config.tip_2x, "2x click") +
             vdc_button(dsuid, vdc_config.tip_3x, "3x click") +
             vdc_button(dsuid, vdc_config.tip_4x, "4x click") +
             "</tr>";

   $("#device_list_content").append(row);

   var callback = function(evt)
   {
       vdc_button_clicked(dsuid, evt);
   };

   $(vdc_button_id(dsuid, vdc_config.tip_1x, true)).click(callback);
   $(vdc_button_id(dsuid, vdc_config.tip_2x, true)).click(callback);
   $(vdc_button_id(dsuid, vdc_config.tip_3x, true)).click(callback);
   $(vdc_button_id(dsuid, vdc_config.tip_4x, true)).click(callback);
};

// some basic initializations on startup
var vdc_setup = function()
{
    // clean potential reload leftovers from the modal window
    window.location.hash = "";

    $("#device_list").hide();

    // clear #close from the URL
    $("a.close").mouseup(
        function()
        {
            // clear #close from the URL
            var t = setTimeout(function()
                {
                    window.location.hash = "";
                    window.clearTimeout(t);
                }, 10);
        });
};

// process getStructure JSON from the dSS and find our vDCs
var vdc_search_callback = function(data)
{
    var all_devices = [];
    var vdc_devices = [];
    var i;

    var was_empty = (Object.keys(g_vdcs).length === 0);

    vdc_clear_search_timeout();

    try
    {
        var zones = data.result.apartment.zones;
        for (i = 0; i < zones.length; i++)
        {
            if (zones[i].id === 0)
            {
                all_devices = zones[i].devices;
                break;
            }
        }
    } catch (err) {}

    for (i = 0; i < all_devices.length; i++)
    {
        var device = all_devices[i];
        if ((device.isPresent === true) && (device.isVdcDevice === true) &&
            (device.VdcModelUID === vdc_config.model_uid))
        {
            vdc_devices.push(device);
        }
    }

    if (vdc_devices.length === 0)
    {
        g_vdcs = {};
        vdc_render_no_devices();
        vdc_start_search_timeout();
        return;
    }

    if (was_empty)
    {
        $("#empty").empty();
        $("#device_list").show();
    }

    jQuery.each(g_vdcs, function(key, val)
        {
            for (i = 0; i < vdc_devices.length; i++)
            {
                if (key === vdc_devices[i].dSUID)
                {
                    return;
                }
            }

            vdc_render_remove_device(key);
            delete g_vdcs[key];
        });

    for (i = 0; i < vdc_devices.length; i++)
    {
        var render = false;
        var vdc = vdc_devices[i];
        if (g_vdcs[vdc.dSUID] == undefined)
        {
            render = true;
        }
        g_vdcs[vdc.dSUID] = vdc;
        if (render)
        {
            vdc_render_add_device(vdc.dSUID);
        }
    }

    if (Object.keys(g_vdcs).length === 0)
    {
        vdc_render_no_devices();
    }

    vdc_start_search_timeout();
};


var vdc_search = function()
{
    vdc_clear_search_timeout();
    // load apartment structure and search for libdsvdc reference vdc's
    $.getJSON("/json/apartment/getStructure",
              vdc_search_callback).fail(vdc_request_failed).fail(vdc_request_failed);
};

$(document).ready(function()
{
    $.ajaxPrefilter(function(options)
    {
        if (!options.beforeSend)
        {
            options.beforeSend = function(xhr)
            {
                var csrf = $.cookie("csrf-token");
                if (csrf)
                {
                    xhr.setRequestHeader('X-CSRF-Token', csrf);
                }
            }
        }
    });
    vdc_setup();
    vdc_search();
});
